@@ -33,17 +33,18 @@ You should have received a copy of the 3-Clause BSD License along with this
 program. If not, see <https://opensource.org/licenses/BSD-3-Clause>
 """
 
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 
 from .detectors import (
-  AGIPD_1MGeometry, AGIPD_500K2GGeometry, DSSC_1MGeometry, JUNGFRAUGeometry,
-  LPD_1MGeometry, PNCCDGeometry, agipd_asic_seams,
+    AGIPD_1MGeometry, AGIPD_500K2GGeometry, GenericGeometry, DSSC_1MGeometry, JUNGFRAUGeometry,
+    LPD_1MGeometry, PNCCDGeometry, agipd_asic_seams,
 )
 
 __all__ = [
     'AGIPD_1MGeometry',
     'AGIPD_500K2GGeometry',
     'agipd_asic_seams',
+    'GenericGeometry',
     'DSSC_1MGeometry',
     'JUNGFRAUGeometry',
     'LPD_1MGeometry',
